@@ -47,6 +47,8 @@ if TYPE_CHECKING:
     from playwright._impl._local_utils import LocalUtils
     from playwright._impl._playwright import Playwright
 
+TimeoutCalculator = Optional[Callable[[Optional[float]], float]]
+
 
 class Channel(AsyncIOEventEmitter):
     def __init__(self, connection: "Connection", object: "ChannelOwner") -> None:
@@ -55,39 +57,68 @@ class Channel(AsyncIOEventEmitter):
         self._guid = object._guid
         self._object = object
         self.on("error", lambda exc: self._connection._on_event_listener_error(exc))
-        self._is_internal_type = False
 
-    async def send(self, method: str, params: Dict = None) -> Any:
+    async def send(
+        self,
+        method: str,
+        timeout_calculator: TimeoutCalculator,
+        params: Dict = None,
+        is_internal: bool = False,
+        title: str = None,
+    ) -> Any:
         return await self._connection.wrap_api_call(
-            lambda: self._inner_send(method, params, False),
-            self._is_internal_type,
+            lambda: self._inner_send(method, timeout_calculator, params, False),
+            is_internal,
+            title,
         )
 
-    async def send_return_as_dict(self, method: str, params: Dict = None) -> Any:
+    async def send_return_as_dict(
+        self,
+        method: str,
+        timeout_calculator: TimeoutCalculator,
+        params: Dict = None,
+        is_internal: bool = False,
+        title: str = None,
+    ) -> Any:
         return await self._connection.wrap_api_call(
-            lambda: self._inner_send(method, params, True),
-            self._is_internal_type,
+            lambda: self._inner_send(method, timeout_calculator, params, True),
+            is_internal,
+            title,
         )
 
-    def send_no_reply(self, method: str, params: Dict = None) -> None:
+    def send_no_reply(
+        self,
+        method: str,
+        timeout_calculator: TimeoutCalculator,
+        params: Dict = None,
+        is_internal: bool = False,
+        title: str = None,
+    ) -> None:
         # No reply messages are used to e.g. waitForEventInfo(after).
         self._connection.wrap_api_call_sync(
             lambda: self._connection._send_message_to_server(
-                self._object, method, {} if params is None else params, True
-            )
+                self._object,
+                method,
+                _augment_params(params, timeout_calculator),
+                True,
+            ),
+            is_internal,
+            title,
         )
 
     async def _inner_send(
-        self, method: str, params: Optional[Dict], return_as_dict: bool
+        self,
+        method: str,
+        timeout_calculator: TimeoutCalculator,
+        params: Optional[Dict],
+        return_as_dict: bool,
     ) -> Any:
-        if params is None:
-            params = {}
         if self._connection._error:
             error = self._connection._error
             self._connection._error = None
             raise error
         callback = self._connection._send_message_to_server(
-            self._object, method, _filter_none(params)
+            self._object, method, _augment_params(params, timeout_calculator)
         )
         done, _ = await asyncio.wait(
             {
@@ -112,9 +143,6 @@ class Channel(AsyncIOEventEmitter):
         key = next(iter(result))
         return result[key]
 
-    def mark_as_internal_type(self) -> None:
-        self._is_internal_type = True
-
 
 class ChannelOwner(AsyncIOEventEmitter):
     def __init__(
@@ -171,7 +199,9 @@ class ChannelOwner(AsyncIOEventEmitter):
         if protocol_event:
             self._connection.wrap_api_call_sync(
                 lambda: self._channel.send_no_reply(
-                    "updateSubscription", {"event": protocol_event, "enabled": enabled}
+                    "updateSubscription",
+                    None,
+                    {"event": protocol_event, "enabled": enabled},
                 ),
                 True,
             )
@@ -188,10 +218,12 @@ class ChannelOwner(AsyncIOEventEmitter):
 
 
 class ProtocolCallback:
-    def __init__(self, loop: asyncio.AbstractEventLoop) -> None:
+    def __init__(self, loop: asyncio.AbstractEventLoop, no_reply: bool = False) -> None:
         self.stack_trace: traceback.StackSummary
-        self.no_reply: bool
+        self.no_reply = no_reply
         self.future = loop.create_future()
+        if no_reply:
+            self.future.set_result(None)
         # The outer task can get cancelled by the user, this forwards the cancellation to the inner task.
         current_task = asyncio.current_task()
 
@@ -218,6 +250,7 @@ class RootChannelOwner(ChannelOwner):
         return from_channel(
             await self._channel.send(
                 "initialize",
+                None,
                 {
                     "sdkLanguage": "python",
                 },
@@ -329,14 +362,13 @@ class Connection(EventEmitter):
             )
         self._last_id += 1
         id = self._last_id
-        callback = ProtocolCallback(self._loop)
+        callback = ProtocolCallback(self._loop, no_reply=no_reply)
         task = asyncio.current_task(self._loop)
         callback.stack_trace = cast(
             traceback.StackSummary,
-            getattr(task, "__pw_stack_trace__", traceback.extract_stack()),
+            getattr(task, "__pw_stack_trace__", traceback.extract_stack(limit=10)),
         )
         callback.no_reply = no_reply
-        self._callbacks[id] = callback
         stack_trace_information = cast(ParsedStackTrace, self._api_zone.get())
         frames = stack_trace_information.get("frames", [])
         location = (
@@ -355,6 +387,9 @@ class Connection(EventEmitter):
         }
         if location:
             metadata["location"] = location  # type: ignore
+        title = stack_trace_information["title"]
+        if title:
+            metadata["title"] = title
         message = {
             "id": id,
             "guid": object._guid,
@@ -362,16 +397,11 @@ class Connection(EventEmitter):
             "params": self._replace_channels_with_guids(params),
             "metadata": metadata,
         }
-        if (
-            self._tracing_count > 0
-            and frames
-            and frames
-            and object._guid != "localUtils"
-        ):
+        if self._tracing_count > 0 and frames and object._guid != "localUtils":
             self.local_utils.add_stack_to_tracing_no_reply(id, frames)
 
-        self._transport.send(message)
         self._callbacks[id] = callback
+        self._transport.send(message)
 
         return callback
 
@@ -392,9 +422,7 @@ class Connection(EventEmitter):
                 parsed_error = parse_error(
                     error["error"], format_call_log(msg.get("log"))  # type: ignore
                 )
-                parsed_error._stack = "".join(
-                    traceback.format_list(callback.stack_trace)[-10:]
-                )
+                parsed_error._stack = "".join(callback.stack_trace.format())
                 callback.future.set_exception(parsed_error)
             else:
                 result = self._replace_guids_with_channels(msg.get("result"))
@@ -514,13 +542,16 @@ class Connection(EventEmitter):
         return payload
 
     async def wrap_api_call(
-        self, cb: Callable[[], Any], is_internal: bool = False
+        self, cb: Callable[[], Any], is_internal: bool = False, title: str = None
     ) -> Any:
         if self._api_zone.get():
             return await cb()
         task = asyncio.current_task(self._loop)
-        st: List[inspect.FrameInfo] = getattr(task, "__pw_stack__", inspect.stack())
-        parsed_st = _extract_stack_trace_information_from_stack(st, is_internal)
+        st: List[inspect.FrameInfo] = getattr(
+            task, "__pw_stack__", None
+        ) or inspect.stack(0)
+
+        parsed_st = _extract_stack_trace_information_from_stack(st, is_internal, title)
         self._api_zone.set(parsed_st)
         try:
             return await cb()
@@ -530,13 +561,15 @@ class Connection(EventEmitter):
             self._api_zone.set(None)
 
     def wrap_api_call_sync(
-        self, cb: Callable[[], Any], is_internal: bool = False
+        self, cb: Callable[[], Any], is_internal: bool = False, title: str = None
     ) -> Any:
         if self._api_zone.get():
             return cb()
         task = asyncio.current_task(self._loop)
-        st: List[inspect.FrameInfo] = getattr(task, "__pw_stack__", inspect.stack())
-        parsed_st = _extract_stack_trace_information_from_stack(st, is_internal)
+        st: List[inspect.FrameInfo] = getattr(
+            task, "__pw_stack__", None
+        ) or inspect.stack(0)
+        parsed_st = _extract_stack_trace_information_from_stack(st, is_internal, title)
         self._api_zone.set(parsed_st)
         try:
             return cb()
@@ -564,10 +597,11 @@ class StackFrame(TypedDict):
 class ParsedStackTrace(TypedDict):
     frames: List[StackFrame]
     apiName: Optional[str]
+    title: Optional[str]
 
 
 def _extract_stack_trace_information_from_stack(
-    st: List[inspect.FrameInfo], is_internal: bool
+    st: List[inspect.FrameInfo], is_internal: bool, title: str = None
 ) -> ParsedStackTrace:
     playwright_module_path = str(Path(playwright.__file__).parents[0])
     last_internal_api_name = ""
@@ -607,11 +641,28 @@ def _extract_stack_trace_information_from_stack(
     return {
         "frames": parsed_frames,
         "apiName": "" if is_internal else api_name,
+        "title": title,
     }
 
 
+def _augment_params(
+    params: Optional[Dict],
+    timeout_calculator: Optional[Callable[[Optional[float]], float]],
+) -> Dict:
+    if params is None:
+        params = {}
+    if timeout_calculator:
+        params["timeout"] = timeout_calculator(params.get("timeout"))
+    return _filter_none(params)
+
+
 def _filter_none(d: Mapping) -> Dict:
-    return {k: v for k, v in d.items() if v is not None}
+    result = {}
+    for k, v in d.items():
+        if v is None:
+            continue
+        result[k] = _filter_none(v) if isinstance(v, dict) else v
+    return result
 
 
 def format_call_log(log: Optional[List[str]]) -> str:
@@ -619,4 +670,4 @@ def format_call_log(log: Optional[List[str]]) -> str:
         return ""
     if len(list(filter(lambda x: x.strip(), log))) == 0:
         return ""
-    return "\nCall log:\n" + "\n  - ".join(log) + "\n"
+    return "\nCall log:\n" + "\n".join(log) + "\n"
