@@ -145,130 +145,140 @@ const EXECUTABLE_PATHS = {
     "win-x64": ["PrintDeps.exe"]
   }
 };
+function cftUrl(suffix) {
+  return ({ browserVersion }) => {
+    return {
+      path: `${browserVersion}/${suffix}`,
+      mirrors: [
+        "https://cdn.playwright.dev/chrome-for-testing-public"
+      ]
+    };
+  };
+}
 const DOWNLOAD_PATHS = {
   "chromium": {
     "<unknown>": void 0,
     "ubuntu18.04-x64": void 0,
-    "ubuntu20.04-x64": "builds/chromium/%s/chromium-linux.zip",
-    "ubuntu22.04-x64": "builds/chromium/%s/chromium-linux.zip",
-    "ubuntu24.04-x64": "builds/chromium/%s/chromium-linux.zip",
+    "ubuntu20.04-x64": cftUrl("linux64/chrome-linux64.zip"),
+    "ubuntu22.04-x64": cftUrl("linux64/chrome-linux64.zip"),
+    "ubuntu24.04-x64": cftUrl("linux64/chrome-linux64.zip"),
     "ubuntu18.04-arm64": void 0,
     "ubuntu20.04-arm64": "builds/chromium/%s/chromium-linux-arm64.zip",
     "ubuntu22.04-arm64": "builds/chromium/%s/chromium-linux-arm64.zip",
     "ubuntu24.04-arm64": "builds/chromium/%s/chromium-linux-arm64.zip",
-    "debian11-x64": "builds/chromium/%s/chromium-linux.zip",
+    "debian11-x64": cftUrl("linux64/chrome-linux64.zip"),
     "debian11-arm64": "builds/chromium/%s/chromium-linux-arm64.zip",
-    "debian12-x64": "builds/chromium/%s/chromium-linux.zip",
+    "debian12-x64": cftUrl("linux64/chrome-linux64.zip"),
     "debian12-arm64": "builds/chromium/%s/chromium-linux-arm64.zip",
-    "debian13-x64": "builds/chromium/%s/chromium-linux.zip",
+    "debian13-x64": cftUrl("linux64/chrome-linux64.zip"),
     "debian13-arm64": "builds/chromium/%s/chromium-linux-arm64.zip",
-    "mac10.13": "builds/chromium/%s/chromium-mac.zip",
-    "mac10.14": "builds/chromium/%s/chromium-mac.zip",
-    "mac10.15": "builds/chromium/%s/chromium-mac.zip",
-    "mac11": "builds/chromium/%s/chromium-mac.zip",
-    "mac11-arm64": "builds/chromium/%s/chromium-mac-arm64.zip",
-    "mac12": "builds/chromium/%s/chromium-mac.zip",
-    "mac12-arm64": "builds/chromium/%s/chromium-mac-arm64.zip",
-    "mac13": "builds/chromium/%s/chromium-mac.zip",
-    "mac13-arm64": "builds/chromium/%s/chromium-mac-arm64.zip",
-    "mac14": "builds/chromium/%s/chromium-mac.zip",
-    "mac14-arm64": "builds/chromium/%s/chromium-mac-arm64.zip",
-    "mac15": "builds/chromium/%s/chromium-mac.zip",
-    "mac15-arm64": "builds/chromium/%s/chromium-mac-arm64.zip",
-    "win64": "builds/chromium/%s/chromium-win64.zip"
+    "mac10.13": cftUrl("mac-x64/chrome-mac-x64.zip"),
+    "mac10.14": cftUrl("mac-x64/chrome-mac-x64.zip"),
+    "mac10.15": cftUrl("mac-x64/chrome-mac-x64.zip"),
+    "mac11": cftUrl("mac-x64/chrome-mac-x64.zip"),
+    "mac11-arm64": cftUrl("mac-arm64/chrome-mac-arm64.zip"),
+    "mac12": cftUrl("mac-x64/chrome-mac-x64.zip"),
+    "mac12-arm64": cftUrl("mac-arm64/chrome-mac-arm64.zip"),
+    "mac13": cftUrl("mac-x64/chrome-mac-x64.zip"),
+    "mac13-arm64": cftUrl("mac-arm64/chrome-mac-arm64.zip"),
+    "mac14": cftUrl("mac-x64/chrome-mac-x64.zip"),
+    "mac14-arm64": cftUrl("mac-arm64/chrome-mac-arm64.zip"),
+    "mac15": cftUrl("mac-x64/chrome-mac-x64.zip"),
+    "mac15-arm64": cftUrl("mac-arm64/chrome-mac-arm64.zip"),
+    "win64": cftUrl("win64/chrome-win64.zip")
   },
   "chromium-headless-shell": {
     "<unknown>": void 0,
     "ubuntu18.04-x64": void 0,
-    "ubuntu20.04-x64": "builds/chromium/%s/chromium-headless-shell-linux.zip",
-    "ubuntu22.04-x64": "builds/chromium/%s/chromium-headless-shell-linux.zip",
-    "ubuntu24.04-x64": "builds/chromium/%s/chromium-headless-shell-linux.zip",
+    "ubuntu20.04-x64": cftUrl("linux64/chrome-headless-shell-linux64.zip"),
+    "ubuntu22.04-x64": cftUrl("linux64/chrome-headless-shell-linux64.zip"),
+    "ubuntu24.04-x64": cftUrl("linux64/chrome-headless-shell-linux64.zip"),
     "ubuntu18.04-arm64": void 0,
     "ubuntu20.04-arm64": "builds/chromium/%s/chromium-headless-shell-linux-arm64.zip",
     "ubuntu22.04-arm64": "builds/chromium/%s/chromium-headless-shell-linux-arm64.zip",
     "ubuntu24.04-arm64": "builds/chromium/%s/chromium-headless-shell-linux-arm64.zip",
-    "debian11-x64": "builds/chromium/%s/chromium-headless-shell-linux.zip",
+    "debian11-x64": cftUrl("linux64/chrome-headless-shell-linux64.zip"),
     "debian11-arm64": "builds/chromium/%s/chromium-headless-shell-linux-arm64.zip",
-    "debian12-x64": "builds/chromium/%s/chromium-headless-shell-linux.zip",
+    "debian12-x64": cftUrl("linux64/chrome-headless-shell-linux64.zip"),
     "debian12-arm64": "builds/chromium/%s/chromium-headless-shell-linux-arm64.zip",
-    "debian13-x64": "builds/chromium/%s/chromium-headless-shell-linux.zip",
+    "debian13-x64": cftUrl("linux64/chrome-headless-shell-linux64.zip"),
     "debian13-arm64": "builds/chromium/%s/chromium-headless-shell-linux-arm64.zip",
     "mac10.13": void 0,
     "mac10.14": void 0,
     "mac10.15": void 0,
-    "mac11": "builds/chromium/%s/chromium-headless-shell-mac.zip",
-    "mac11-arm64": "builds/chromium/%s/chromium-headless-shell-mac-arm64.zip",
-    "mac12": "builds/chromium/%s/chromium-headless-shell-mac.zip",
-    "mac12-arm64": "builds/chromium/%s/chromium-headless-shell-mac-arm64.zip",
-    "mac13": "builds/chromium/%s/chromium-headless-shell-mac.zip",
-    "mac13-arm64": "builds/chromium/%s/chromium-headless-shell-mac-arm64.zip",
-    "mac14": "builds/chromium/%s/chromium-headless-shell-mac.zip",
-    "mac14-arm64": "builds/chromium/%s/chromium-headless-shell-mac-arm64.zip",
-    "mac15": "builds/chromium/%s/chromium-headless-shell-mac.zip",
-    "mac15-arm64": "builds/chromium/%s/chromium-headless-shell-mac-arm64.zip",
-    "win64": "builds/chromium/%s/chromium-headless-shell-win64.zip"
+    "mac11": cftUrl("mac-x64/chrome-headless-shell-mac-x64.zip"),
+    "mac11-arm64": cftUrl("mac-arm64/chrome-headless-shell-mac-arm64.zip"),
+    "mac12": cftUrl("mac-x64/chrome-headless-shell-mac-x64.zip"),
+    "mac12-arm64": cftUrl("mac-arm64/chrome-headless-shell-mac-arm64.zip"),
+    "mac13": cftUrl("mac-x64/chrome-headless-shell-mac-x64.zip"),
+    "mac13-arm64": cftUrl("mac-arm64/chrome-headless-shell-mac-arm64.zip"),
+    "mac14": cftUrl("mac-x64/chrome-headless-shell-mac-x64.zip"),
+    "mac14-arm64": cftUrl("mac-arm64/chrome-headless-shell-mac-arm64.zip"),
+    "mac15": cftUrl("mac-x64/chrome-headless-shell-mac-x64.zip"),
+    "mac15-arm64": cftUrl("mac-arm64/chrome-headless-shell-mac-arm64.zip"),
+    "win64": cftUrl("win64/chrome-headless-shell-win64.zip")
   },
   "chromium-tip-of-tree": {
     "<unknown>": void 0,
     "ubuntu18.04-x64": void 0,
-    "ubuntu20.04-x64": "builds/chromium-tip-of-tree/%s/chromium-tip-of-tree-linux.zip",
-    "ubuntu22.04-x64": "builds/chromium-tip-of-tree/%s/chromium-tip-of-tree-linux.zip",
-    "ubuntu24.04-x64": "builds/chromium-tip-of-tree/%s/chromium-tip-of-tree-linux.zip",
+    "ubuntu20.04-x64": cftUrl("linux64/chrome-linux64.zip"),
+    "ubuntu22.04-x64": cftUrl("linux64/chrome-linux64.zip"),
+    "ubuntu24.04-x64": cftUrl("linux64/chrome-linux64.zip"),
     "ubuntu18.04-arm64": void 0,
     "ubuntu20.04-arm64": "builds/chromium-tip-of-tree/%s/chromium-tip-of-tree-linux-arm64.zip",
     "ubuntu22.04-arm64": "builds/chromium-tip-of-tree/%s/chromium-tip-of-tree-linux-arm64.zip",
     "ubuntu24.04-arm64": "builds/chromium-tip-of-tree/%s/chromium-tip-of-tree-linux-arm64.zip",
-    "debian11-x64": "builds/chromium-tip-of-tree/%s/chromium-tip-of-tree-linux.zip",
+    "debian11-x64": cftUrl("linux64/chrome-linux64.zip"),
     "debian11-arm64": "builds/chromium-tip-of-tree/%s/chromium-tip-of-tree-linux-arm64.zip",
-    "debian12-x64": "builds/chromium-tip-of-tree/%s/chromium-tip-of-tree-linux.zip",
+    "debian12-x64": cftUrl("linux64/chrome-linux64.zip"),
     "debian12-arm64": "builds/chromium-tip-of-tree/%s/chromium-tip-of-tree-linux-arm64.zip",
-    "debian13-x64": "builds/chromium-tip-of-tree/%s/chromium-tip-of-tree-linux.zip",
+    "debian13-x64": cftUrl("linux64/chrome-linux64.zip"),
     "debian13-arm64": "builds/chromium-tip-of-tree/%s/chromium-tip-of-tree-linux-arm64.zip",
-    "mac10.13": "builds/chromium-tip-of-tree/%s/chromium-tip-of-tree-mac.zip",
-    "mac10.14": "builds/chromium-tip-of-tree/%s/chromium-tip-of-tree-mac.zip",
-    "mac10.15": "builds/chromium-tip-of-tree/%s/chromium-tip-of-tree-mac.zip",
-    "mac11": "builds/chromium-tip-of-tree/%s/chromium-tip-of-tree-mac.zip",
-    "mac11-arm64": "builds/chromium-tip-of-tree/%s/chromium-tip-of-tree-mac-arm64.zip",
-    "mac12": "builds/chromium-tip-of-tree/%s/chromium-tip-of-tree-mac.zip",
-    "mac12-arm64": "builds/chromium-tip-of-tree/%s/chromium-tip-of-tree-mac-arm64.zip",
-    "mac13": "builds/chromium-tip-of-tree/%s/chromium-tip-of-tree-mac.zip",
-    "mac13-arm64": "builds/chromium-tip-of-tree/%s/chromium-tip-of-tree-mac-arm64.zip",
-    "mac14": "builds/chromium-tip-of-tree/%s/chromium-tip-of-tree-mac.zip",
-    "mac14-arm64": "builds/chromium-tip-of-tree/%s/chromium-tip-of-tree-mac-arm64.zip",
-    "mac15": "builds/chromium-tip-of-tree/%s/chromium-tip-of-tree-mac.zip",
-    "mac15-arm64": "builds/chromium-tip-of-tree/%s/chromium-tip-of-tree-mac-arm64.zip",
-    "win64": "builds/chromium-tip-of-tree/%s/chromium-tip-of-tree-win64.zip"
+    "mac10.13": cftUrl("mac-x64/chrome-mac-x64.zip"),
+    "mac10.14": cftUrl("mac-x64/chrome-mac-x64.zip"),
+    "mac10.15": cftUrl("mac-x64/chrome-mac-x64.zip"),
+    "mac11": cftUrl("mac-x64/chrome-mac-x64.zip"),
+    "mac11-arm64": cftUrl("mac-arm64/chrome-mac-arm64.zip"),
+    "mac12": cftUrl("mac-x64/chrome-mac-x64.zip"),
+    "mac12-arm64": cftUrl("mac-arm64/chrome-mac-arm64.zip"),
+    "mac13": cftUrl("mac-x64/chrome-mac-x64.zip"),
+    "mac13-arm64": cftUrl("mac-arm64/chrome-mac-arm64.zip"),
+    "mac14": cftUrl("mac-x64/chrome-mac-x64.zip"),
+    "mac14-arm64": cftUrl("mac-arm64/chrome-mac-arm64.zip"),
+    "mac15": cftUrl("mac-x64/chrome-mac-x64.zip"),
+    "mac15-arm64": cftUrl("mac-arm64/chrome-mac-arm64.zip"),
+    "win64": cftUrl("win64/chrome-win64.zip")
   },
   "chromium-tip-of-tree-headless-shell": {
     "<unknown>": void 0,
     "ubuntu18.04-x64": void 0,
-    "ubuntu20.04-x64": "builds/chromium-tip-of-tree/%s/chromium-tip-of-tree-headless-shell-linux.zip",
-    "ubuntu22.04-x64": "builds/chromium-tip-of-tree/%s/chromium-tip-of-tree-headless-shell-linux.zip",
-    "ubuntu24.04-x64": "builds/chromium-tip-of-tree/%s/chromium-tip-of-tree-headless-shell-linux.zip",
+    "ubuntu20.04-x64": cftUrl("linux64/chrome-headless-shell-linux64.zip"),
+    "ubuntu22.04-x64": cftUrl("linux64/chrome-headless-shell-linux64.zip"),
+    "ubuntu24.04-x64": cftUrl("linux64/chrome-headless-shell-linux64.zip"),
     "ubuntu18.04-arm64": void 0,
     "ubuntu20.04-arm64": "builds/chromium-tip-of-tree/%s/chromium-tip-of-tree-headless-shell-linux-arm64.zip",
     "ubuntu22.04-arm64": "builds/chromium-tip-of-tree/%s/chromium-tip-of-tree-headless-shell-linux-arm64.zip",
     "ubuntu24.04-arm64": "builds/chromium-tip-of-tree/%s/chromium-tip-of-tree-headless-shell-linux-arm64.zip",
-    "debian11-x64": "builds/chromium-tip-of-tree/%s/chromium-tip-of-tree-headless-shell-linux.zip",
+    "debian11-x64": cftUrl("linux64/chrome-headless-shell-linux64.zip"),
     "debian11-arm64": "builds/chromium-tip-of-tree/%s/chromium-tip-of-tree-headless-shell-linux-arm64.zip",
-    "debian12-x64": "builds/chromium-tip-of-tree/%s/chromium-tip-of-tree-headless-shell-linux.zip",
+    "debian12-x64": cftUrl("linux64/chrome-headless-shell-linux64.zip"),
     "debian12-arm64": "builds/chromium-tip-of-tree/%s/chromium-tip-of-tree-headless-shell-linux-arm64.zip",
-    "debian13-x64": "builds/chromium-tip-of-tree/%s/chromium-tip-of-tree-headless-shell-linux.zip",
+    "debian13-x64": cftUrl("linux64/chrome-headless-shell-linux64.zip"),
     "debian13-arm64": "builds/chromium-tip-of-tree/%s/chromium-tip-of-tree-headless-shell-linux-arm64.zip",
     "mac10.13": void 0,
     "mac10.14": void 0,
     "mac10.15": void 0,
-    "mac11": "builds/chromium-tip-of-tree/%s/chromium-tip-of-tree-headless-shell-mac.zip",
-    "mac11-arm64": "builds/chromium-tip-of-tree/%s/chromium-tip-of-tree-headless-shell-mac-arm64.zip",
-    "mac12": "builds/chromium-tip-of-tree/%s/chromium-tip-of-tree-headless-shell-mac.zip",
-    "mac12-arm64": "builds/chromium-tip-of-tree/%s/chromium-tip-of-tree-headless-shell-mac-arm64.zip",
-    "mac13": "builds/chromium-tip-of-tree/%s/chromium-tip-of-tree-headless-shell-mac.zip",
-    "mac13-arm64": "builds/chromium-tip-of-tree/%s/chromium-tip-of-tree-headless-shell-mac-arm64.zip",
-    "mac14": "builds/chromium-tip-of-tree/%s/chromium-tip-of-tree-headless-shell-mac.zip",
-    "mac14-arm64": "builds/chromium-tip-of-tree/%s/chromium-tip-of-tree-headless-shell-mac-arm64.zip",
-    "mac15": "builds/chromium-tip-of-tree/%s/chromium-tip-of-tree-headless-shell-mac.zip",
-    "mac15-arm64": "builds/chromium-tip-of-tree/%s/chromium-tip-of-tree-headless-shell-mac-arm64.zip",
-    "win64": "builds/chromium-tip-of-tree/%s/chromium-tip-of-tree-headless-shell-win64.zip"
+    "mac11": cftUrl("mac-x64/chrome-headless-shell-mac-x64.zip"),
+    "mac11-arm64": cftUrl("mac-arm64/chrome-headless-shell-mac-arm64.zip"),
+    "mac12": cftUrl("mac-x64/chrome-headless-shell-mac-x64.zip"),
+    "mac12-arm64": cftUrl("mac-arm64/chrome-headless-shell-mac-arm64.zip"),
+    "mac13": cftUrl("mac-x64/chrome-headless-shell-mac-x64.zip"),
+    "mac13-arm64": cftUrl("mac-arm64/chrome-headless-shell-mac-arm64.zip"),
+    "mac14": cftUrl("mac-x64/chrome-headless-shell-mac-x64.zip"),
+    "mac14-arm64": cftUrl("mac-arm64/chrome-headless-shell-mac-arm64.zip"),
+    "mac15": cftUrl("mac-x64/chrome-headless-shell-mac-x64.zip"),
+    "mac15-arm64": cftUrl("mac-arm64/chrome-headless-shell-mac-arm64.zip"),
+    "win64": cftUrl("win64/chrome-headless-shell-win64.zip")
   },
   "firefox": {
     "<unknown>": void 0,
@@ -349,14 +359,14 @@ const DOWNLOAD_PATHS = {
     "debian13-x64": "builds/webkit/%s/webkit-debian-13.zip",
     "debian13-arm64": "builds/webkit/%s/webkit-debian-13-arm64.zip",
     "mac10.13": void 0,
-    "mac10.14": "builds/deprecated-webkit-mac-10.14/%s/deprecated-webkit-mac-10.14.zip",
-    "mac10.15": "builds/deprecated-webkit-mac-10.15/%s/deprecated-webkit-mac-10.15.zip",
-    "mac11": "builds/webkit/%s/webkit-mac-11.zip",
-    "mac11-arm64": "builds/webkit/%s/webkit-mac-11-arm64.zip",
-    "mac12": "builds/webkit/%s/webkit-mac-12.zip",
-    "mac12-arm64": "builds/webkit/%s/webkit-mac-12-arm64.zip",
-    "mac13": "builds/webkit/%s/webkit-mac-13.zip",
-    "mac13-arm64": "builds/webkit/%s/webkit-mac-13-arm64.zip",
+    "mac10.14": void 0,
+    "mac10.15": void 0,
+    "mac11": void 0,
+    "mac11-arm64": void 0,
+    "mac12": void 0,
+    "mac12-arm64": void 0,
+    "mac13": void 0,
+    "mac13-arm64": void 0,
     "mac14": "builds/webkit/%s/webkit-mac-14.zip",
     "mac14-arm64": "builds/webkit/%s/webkit-mac-14-arm64.zip",
     "mac15": "builds/webkit/%s/webkit-mac-15.zip",
@@ -483,7 +493,7 @@ const registryDirectory = (() => {
 })();
 function isBrowserDirectory(browserDirectory) {
   const baseName = import_path.default.basename(browserDirectory);
-  for (const browserName of allDownloadable) {
+  for (const browserName of allDownloadableDirectoriesThatEverExisted) {
     if (baseName.startsWith(browserName.replace(/-/g, "_") + "-"))
       return true;
   }
@@ -501,6 +511,7 @@ function readDescriptors(browsersJSON) {
       hasRevisionOverride: !!revisionOverride,
       // We only put browser version for the supported operating systems.
       browserVersion: revisionOverride ? void 0 : obj.browserVersion,
+      title: obj["title"],
       installByDefault: !!obj.installByDefault,
       // Method `isBrowserDirectory` determines directory to be browser iff
       // it starts with some browser name followed by '-'. Some browser names
@@ -512,7 +523,8 @@ function readDescriptors(browsersJSON) {
     return descriptor;
   });
 }
-const allDownloadable = ["android", "chromium", "firefox", "webkit", "ffmpeg", "firefox-beta", "chromium-tip-of-tree", "chromium-headless-shell", "chromium-tip-of-tree-headless-shell"];
+const allDownloadableDirectoriesThatEverExisted = ["android", "chromium", "firefox", "webkit", "ffmpeg", "firefox-beta", "chromium-tip-of-tree", "chromium-headless-shell", "chromium-tip-of-tree-headless-shell", "winldd"];
+const chromiumAliases = ["bidi-chromium", "chrome-for-testing"];
 class Registry {
   constructor(browsersJSON) {
     const descriptors = readDescriptors(browsersJSON);
@@ -552,7 +564,6 @@ ${(0, import_ascii.wrapInASCIIBox)(prettyMessage, 1)}`);
     const chromium = descriptors.find((d) => d.name === "chromium");
     const chromiumExecutable = findExecutablePath(chromium.dir, "chromium");
     this._executables.push({
-      type: "browser",
       name: "chromium",
       browserName: "chromium",
       directory: chromium.dir,
@@ -561,15 +572,16 @@ ${(0, import_ascii.wrapInASCIIBox)(prettyMessage, 1)}`);
       installType: chromium.installByDefault ? "download-by-default" : "download-on-demand",
       _validateHostRequirements: (sdkLanguage) => this._validateHostRequirements(sdkLanguage, chromium.dir, ["chrome-linux"], [], ["chrome-win"]),
       downloadURLs: this._downloadURLs(chromium),
+      title: chromium.title,
+      revision: chromium.revision,
       browserVersion: chromium.browserVersion,
-      _install: () => this._downloadExecutable(chromium, chromiumExecutable),
+      _install: (force) => this._downloadExecutable(chromium, force, chromiumExecutable),
       _dependencyGroup: "chromium",
       _isHermeticInstallation: true
     });
     const chromiumHeadlessShell = descriptors.find((d) => d.name === "chromium-headless-shell");
     const chromiumHeadlessShellExecutable = findExecutablePath(chromiumHeadlessShell.dir, "chromium-headless-shell");
     this._executables.push({
-      type: "channel",
       name: "chromium-headless-shell",
       browserName: "chromium",
       directory: chromiumHeadlessShell.dir,
@@ -578,15 +590,16 @@ ${(0, import_ascii.wrapInASCIIBox)(prettyMessage, 1)}`);
       installType: chromiumHeadlessShell.installByDefault ? "download-by-default" : "download-on-demand",
       _validateHostRequirements: (sdkLanguage) => this._validateHostRequirements(sdkLanguage, chromiumHeadlessShell.dir, ["chrome-linux"], [], ["chrome-win"]),
       downloadURLs: this._downloadURLs(chromiumHeadlessShell),
-      browserVersion: chromium.browserVersion,
-      _install: () => this._downloadExecutable(chromiumHeadlessShell, chromiumHeadlessShellExecutable),
+      title: chromiumHeadlessShell.title,
+      revision: chromiumHeadlessShell.revision,
+      browserVersion: chromiumHeadlessShell.browserVersion,
+      _install: (force) => this._downloadExecutable(chromiumHeadlessShell, force, chromiumHeadlessShellExecutable),
       _dependencyGroup: "chromium",
       _isHermeticInstallation: true
     });
     const chromiumTipOfTreeHeadlessShell = descriptors.find((d) => d.name === "chromium-tip-of-tree-headless-shell");
     const chromiumTipOfTreeHeadlessShellExecutable = findExecutablePath(chromiumTipOfTreeHeadlessShell.dir, "chromium-tip-of-tree-headless-shell");
     this._executables.push({
-      type: "channel",
       name: "chromium-tip-of-tree-headless-shell",
       browserName: "chromium",
       directory: chromiumTipOfTreeHeadlessShell.dir,
@@ -595,15 +608,16 @@ ${(0, import_ascii.wrapInASCIIBox)(prettyMessage, 1)}`);
       installType: chromiumTipOfTreeHeadlessShell.installByDefault ? "download-by-default" : "download-on-demand",
       _validateHostRequirements: (sdkLanguage) => this._validateHostRequirements(sdkLanguage, chromiumTipOfTreeHeadlessShell.dir, ["chrome-linux"], [], ["chrome-win"]),
       downloadURLs: this._downloadURLs(chromiumTipOfTreeHeadlessShell),
-      browserVersion: chromium.browserVersion,
-      _install: () => this._downloadExecutable(chromiumTipOfTreeHeadlessShell, chromiumTipOfTreeHeadlessShellExecutable),
+      title: chromiumTipOfTreeHeadlessShell.title,
+      revision: chromiumTipOfTreeHeadlessShell.revision,
+      browserVersion: chromiumTipOfTreeHeadlessShell.browserVersion,
+      _install: (force) => this._downloadExecutable(chromiumTipOfTreeHeadlessShell, force, chromiumTipOfTreeHeadlessShellExecutable),
       _dependencyGroup: "chromium",
       _isHermeticInstallation: true
     });
     const chromiumTipOfTree = descriptors.find((d) => d.name === "chromium-tip-of-tree");
     const chromiumTipOfTreeExecutable = findExecutablePath(chromiumTipOfTree.dir, "chromium-tip-of-tree");
     this._executables.push({
-      type: "tool",
       name: "chromium-tip-of-tree",
       browserName: "chromium",
       directory: chromiumTipOfTree.dir,
@@ -612,8 +626,10 @@ ${(0, import_ascii.wrapInASCIIBox)(prettyMessage, 1)}`);
       installType: chromiumTipOfTree.installByDefault ? "download-by-default" : "download-on-demand",
       _validateHostRequirements: (sdkLanguage) => this._validateHostRequirements(sdkLanguage, chromiumTipOfTree.dir, ["chrome-linux"], [], ["chrome-win"]),
       downloadURLs: this._downloadURLs(chromiumTipOfTree),
+      title: chromiumTipOfTree.title,
+      revision: chromiumTipOfTree.revision,
       browserVersion: chromiumTipOfTree.browserVersion,
-      _install: () => this._downloadExecutable(chromiumTipOfTree, chromiumTipOfTreeExecutable),
+      _install: (force) => this._downloadExecutable(chromiumTipOfTree, force, chromiumTipOfTreeExecutable),
       _dependencyGroup: "chromium",
       _isHermeticInstallation: true
     });
@@ -702,25 +718,9 @@ ${(0, import_ascii.wrapInASCIIBox)(prettyMessage, 1)}`);
       "darwin": "/Applications/Google Chrome Canary.app/Contents/MacOS/Google Chrome Canary",
       "win32": `\\Google\\Chrome SxS\\Application\\chrome.exe`
     }));
-    this._executables.push({
-      type: "channel",
-      name: "bidi-chromium",
-      browserName: "chromium",
-      directory: chromium.dir,
-      executablePath: () => chromiumExecutable,
-      executablePathOrDie: (sdkLanguage) => executablePathOrDie("chromium", chromiumExecutable, chromium.installByDefault, sdkLanguage),
-      installType: "download-on-demand",
-      _validateHostRequirements: (sdkLanguage) => this._validateHostRequirements(sdkLanguage, chromium.dir, ["chrome-linux"], [], ["chrome-win"]),
-      downloadURLs: this._downloadURLs(chromium),
-      browserVersion: chromium.browserVersion,
-      _install: () => this._downloadExecutable(chromium, chromiumExecutable),
-      _dependencyGroup: "chromium",
-      _isHermeticInstallation: true
-    });
     const firefox = descriptors.find((d) => d.name === "firefox");
     const firefoxExecutable = findExecutablePath(firefox.dir, "firefox");
     this._executables.push({
-      type: "browser",
       name: "firefox",
       browserName: "firefox",
       directory: firefox.dir,
@@ -729,15 +729,16 @@ ${(0, import_ascii.wrapInASCIIBox)(prettyMessage, 1)}`);
       installType: firefox.installByDefault ? "download-by-default" : "download-on-demand",
       _validateHostRequirements: (sdkLanguage) => this._validateHostRequirements(sdkLanguage, firefox.dir, ["firefox"], [], ["firefox"]),
       downloadURLs: this._downloadURLs(firefox),
+      title: firefox.title,
+      revision: firefox.revision,
       browserVersion: firefox.browserVersion,
-      _install: () => this._downloadExecutable(firefox, firefoxExecutable),
+      _install: (force) => this._downloadExecutable(firefox, force, firefoxExecutable),
       _dependencyGroup: "firefox",
       _isHermeticInstallation: true
     });
     const firefoxBeta = descriptors.find((d) => d.name === "firefox-beta");
     const firefoxBetaExecutable = findExecutablePath(firefoxBeta.dir, "firefox");
     this._executables.push({
-      type: "tool",
       name: "firefox-beta",
       browserName: "firefox",
       directory: firefoxBeta.dir,
@@ -746,8 +747,10 @@ ${(0, import_ascii.wrapInASCIIBox)(prettyMessage, 1)}`);
       installType: firefoxBeta.installByDefault ? "download-by-default" : "download-on-demand",
       _validateHostRequirements: (sdkLanguage) => this._validateHostRequirements(sdkLanguage, firefoxBeta.dir, ["firefox"], [], ["firefox"]),
       downloadURLs: this._downloadURLs(firefoxBeta),
+      title: firefoxBeta.title,
+      revision: firefoxBeta.revision,
       browserVersion: firefoxBeta.browserVersion,
-      _install: () => this._downloadExecutable(firefoxBeta, firefoxBetaExecutable),
+      _install: (force) => this._downloadExecutable(firefoxBeta, force, firefoxBetaExecutable),
       _dependencyGroup: "firefox",
       _isHermeticInstallation: true
     });
@@ -764,7 +767,6 @@ ${(0, import_ascii.wrapInASCIIBox)(prettyMessage, 1)}`);
       import_path.default.join("minibrowser-wpe", "sys", "lib")
     ];
     this._executables.push({
-      type: "browser",
       name: "webkit",
       browserName: "webkit",
       directory: webkit.dir,
@@ -773,19 +775,21 @@ ${(0, import_ascii.wrapInASCIIBox)(prettyMessage, 1)}`);
       installType: webkit.installByDefault ? "download-by-default" : "download-on-demand",
       _validateHostRequirements: (sdkLanguage) => this._validateHostRequirements(sdkLanguage, webkit.dir, webkitLinuxLddDirectories, ["libGLESv2.so.2", "libx264.so"], [""]),
       downloadURLs: this._downloadURLs(webkit),
+      title: webkit.title,
+      revision: webkit.revision,
       browserVersion: webkit.browserVersion,
-      _install: () => this._downloadExecutable(webkit, webkitExecutable),
+      _install: (force) => this._downloadExecutable(webkit, force, webkitExecutable),
       _dependencyGroup: "webkit",
       _isHermeticInstallation: true
     });
     this._executables.push({
-      type: "channel",
       name: "webkit-wsl",
       browserName: "webkit",
       directory: webkit.dir,
       executablePath: () => webkitExecutable,
       executablePathOrDie: (sdkLanguage) => executablePathOrDie("webkit", webkitExecutable, webkit.installByDefault, sdkLanguage),
       installType: "download-on-demand",
+      title: "Webkit in WSL",
       _validateHostRequirements: (sdkLanguage) => Promise.resolve(),
       _isHermeticInstallation: true,
       _install: async () => {
@@ -807,7 +811,6 @@ ${(0, import_ascii.wrapInASCIIBox)(prettyMessage, 1)}`);
     const ffmpeg = descriptors.find((d) => d.name === "ffmpeg");
     const ffmpegExecutable = findExecutablePath(ffmpeg.dir, "ffmpeg");
     this._executables.push({
-      type: "tool",
       name: "ffmpeg",
       browserName: void 0,
       directory: ffmpeg.dir,
@@ -816,14 +819,15 @@ ${(0, import_ascii.wrapInASCIIBox)(prettyMessage, 1)}`);
       installType: ffmpeg.installByDefault ? "download-by-default" : "download-on-demand",
       _validateHostRequirements: () => Promise.resolve(),
       downloadURLs: this._downloadURLs(ffmpeg),
-      _install: () => this._downloadExecutable(ffmpeg, ffmpegExecutable),
+      title: ffmpeg.title,
+      revision: ffmpeg.revision,
+      _install: (force) => this._downloadExecutable(ffmpeg, force, ffmpegExecutable),
       _dependencyGroup: "tools",
       _isHermeticInstallation: true
     });
     const winldd = descriptors.find((d) => d.name === "winldd");
     const winlddExecutable = findExecutablePath(winldd.dir, "winldd");
     this._executables.push({
-      type: "tool",
       name: "winldd",
       browserName: void 0,
       directory: winldd.dir,
@@ -832,13 +836,14 @@ ${(0, import_ascii.wrapInASCIIBox)(prettyMessage, 1)}`);
       installType: process.platform === "win32" ? "download-by-default" : "none",
       _validateHostRequirements: () => Promise.resolve(),
       downloadURLs: this._downloadURLs(winldd),
-      _install: () => this._downloadExecutable(winldd, winlddExecutable),
+      title: winldd.title,
+      revision: winldd.revision,
+      _install: (force) => this._downloadExecutable(winldd, force, winlddExecutable),
       _dependencyGroup: "tools",
       _isHermeticInstallation: true
     });
     const android = descriptors.find((d) => d.name === "android");
     this._executables.push({
-      type: "tool",
       name: "android",
       browserName: void 0,
       directory: android.dir,
@@ -847,7 +852,9 @@ ${(0, import_ascii.wrapInASCIIBox)(prettyMessage, 1)}`);
       installType: "download-on-demand",
       _validateHostRequirements: () => Promise.resolve(),
       downloadURLs: this._downloadURLs(android),
-      _install: () => this._downloadExecutable(android),
+      title: android.title,
+      revision: android.revision,
+      _install: (force) => this._downloadExecutable(android, force),
       _dependencyGroup: "tools",
       _isHermeticInstallation: true
     });
@@ -881,7 +888,6 @@ Run "${buildPlaywrightCLICommand(sdkLanguage, "install " + name)}"` : "";
       throw new Error(`Chromium distribution '${name}' is not found${location}${installation}`);
     };
     return {
-      type: "channel",
       name,
       browserName: "chromium",
       directory: void 0,
@@ -920,7 +926,6 @@ Run "${buildPlaywrightCLICommand(sdkLanguage, "install " + name)}"` : "";
       return void 0;
     };
     return {
-      type: "channel",
       name,
       browserName: "firefox",
       directory: void 0,
@@ -932,7 +937,7 @@ Run "${buildPlaywrightCLICommand(sdkLanguage, "install " + name)}"` : "";
       _install: install
     };
   }
-  _createBidiChromiumChannel(name, lookAt, install) {
+  _createBidiChromiumChannel(name, lookAt) {
     const executablePath = (sdkLanguage, shouldThrow) => {
       const suffix = lookAt[process.platform];
       if (!suffix) {
@@ -956,21 +961,17 @@ Run "${buildPlaywrightCLICommand(sdkLanguage, "install " + name)}"` : "";
       if (!shouldThrow)
         return void 0;
       const location = prefixes.length ? ` at ${import_path.default.join(prefixes[0], suffix)}` : ``;
-      const installation = install ? `
-Run "${buildPlaywrightCLICommand(sdkLanguage, "install " + name)}"` : "";
-      throw new Error(`Chromium distribution '${name}' is not found${location}${installation}`);
+      throw new Error(`Chromium distribution '${name}' is not found${location}`);
     };
     return {
-      type: "channel",
       name,
       browserName: "chromium",
       directory: void 0,
       executablePath: (sdkLanguage) => executablePath(sdkLanguage, false),
       executablePathOrDie: (sdkLanguage) => executablePath(sdkLanguage, true),
-      installType: install ? "install-script" : "none",
+      installType: "none",
       _validateHostRequirements: () => Promise.resolve(),
-      _isHermeticInstallation: false,
-      _install: install
+      _isHermeticInstallation: false
     };
   }
   executables() {
@@ -1051,7 +1052,7 @@ Run "${buildPlaywrightCLICommand(sdkLanguage, "install " + name)}"` : "";
           ].join("\n"), 1) + "\n\n");
           return;
         }
-        await executable._install();
+        await executable._install(!!options?.force);
       }
     } catch (e) {
       if (e.code === "ELOCKED") {
@@ -1123,8 +1124,16 @@ Run "${buildPlaywrightCLICommand(sdkLanguage, "install " + name)}"` : "";
     const downloadPathTemplate = paths[import_hostPlatform.hostPlatform] || paths["<unknown>"];
     if (!downloadPathTemplate)
       return [];
-    const downloadPath = util.format(downloadPathTemplate, descriptor.revision);
-    let downloadURLs = PLAYWRIGHT_CDN_MIRRORS.map((mirror) => `${mirror}/${downloadPath}`);
+    let downloadPath;
+    let mirrors;
+    if (typeof downloadPathTemplate === "function") {
+      const result = downloadPathTemplate(descriptor);
+      downloadPath = result.path;
+      mirrors = result.mirrors;
+    } else {
+      downloadPath = util.format(downloadPathTemplate, descriptor.revision);
+      mirrors = PLAYWRIGHT_CDN_MIRRORS;
+    }
     let downloadHostEnv;
     if (descriptor.name.startsWith("chromium"))
       downloadHostEnv = "PLAYWRIGHT_CHROMIUM_DOWNLOAD_HOST";
@@ -1134,10 +1143,10 @@ Run "${buildPlaywrightCLICommand(sdkLanguage, "install " + name)}"` : "";
       downloadHostEnv = "PLAYWRIGHT_WEBKIT_DOWNLOAD_HOST";
     const customHostOverride = downloadHostEnv && (0, import_utils.getFromENV)(downloadHostEnv) || (0, import_utils.getFromENV)("PLAYWRIGHT_DOWNLOAD_HOST");
     if (customHostOverride)
-      downloadURLs = [`${customHostOverride}/${downloadPath}`];
-    return downloadURLs;
+      mirrors = [customHostOverride];
+    return mirrors.map((mirror) => `${mirror}/${downloadPath}`);
   }
-  async _downloadExecutable(descriptor, executablePath) {
+  async _downloadExecutable(descriptor, force, executablePath) {
     const downloadURLs = this._downloadURLs(descriptor);
     if (!downloadURLs.length)
       throw new Error(`ERROR: Playwright does not support ${descriptor.name} on ${import_hostPlatform.hostPlatform}`);
@@ -1150,18 +1159,22 @@ Run "${buildPlaywrightCLICommand(sdkLanguage, "install " + name)}"` : "";
       else
         (0, import_browserFetcher.logPolitely)(message);
     }
-    const displayName = descriptor.name.split("-").map((word) => {
-      return word === "ffmpeg" ? "FFMPEG" : word.charAt(0).toUpperCase() + word.slice(1);
-    }).join(" ");
-    const title = descriptor.browserVersion ? `${displayName} ${descriptor.browserVersion} (playwright build v${descriptor.revision})` : `${displayName} playwright build v${descriptor.revision}`;
+    const title = this.calculateDownloadTitle(descriptor);
     const downloadFileName = `playwright-download-${descriptor.name}-${import_hostPlatform.hostPlatform}-${descriptor.revision}.zip`;
     const downloadSocketTimeoutEnv = (0, import_utils.getFromENV)("PLAYWRIGHT_DOWNLOAD_CONNECTION_TIMEOUT");
     const downloadSocketTimeout = +(downloadSocketTimeoutEnv || "0") || import_network.NET_DEFAULT_TIMEOUT;
-    await (0, import_browserFetcher.downloadBrowserWithProgressBar)(title, descriptor.dir, executablePath, downloadURLs, downloadFileName, downloadSocketTimeout).catch((e) => {
+    await (0, import_browserFetcher.downloadBrowserWithProgressBar)(title, descriptor.dir, executablePath, downloadURLs, downloadFileName, downloadSocketTimeout, force).catch((e) => {
       throw new Error(`Failed to download ${title}, caused by
 ${e.stack}`);
     });
   }
+  calculateDownloadTitle(descriptor) {
+    const title = descriptor.title ?? descriptor.name.split("-").map((word) => {
+      return word === "ffmpeg" ? "FFmpeg" : word.charAt(0).toUpperCase() + word.slice(1);
+    }).join(" ");
+    const version = descriptor.browserVersion ? " " + descriptor.browserVersion : "";
+    return `${title}${version} (playwright ${descriptor.name} v${descriptor.revision})`;
+  }
   async _installMSEdgeChannel(channel, scripts) {
     const scriptArgs = [];
     if (process.platform !== "linux") {
@@ -1234,7 +1247,7 @@ ${e.stack}`);
         linkTarget = (await import_fs.default.promises.readFile(linkPath)).toString();
         const browsersJSON = require(import_path.default.join(linkTarget, "browsers.json"));
         const descriptors = readDescriptors(browsersJSON);
-        for (const browserName of allDownloadable) {
+        for (const browserName of allDownloadableDirectoriesThatEverExisted) {
           const descriptor = descriptors.find((d) => d.name === browserName);
           if (!descriptor)
             continue;
@@ -1279,13 +1292,18 @@ ${e.stack}`);
   _defaultBrowsersToInstall(options) {
     let executables = this.defaultExecutables();
     if (options.shell === "no")
-      executables = executables.filter((e) => e.name !== "chromium-headless-shell");
+      executables = executables.filter((e) => e.name !== "chromium-headless-shell" && e.name !== "chromium-tip-of-tree-headless-shell");
     if (options.shell === "only")
-      executables = executables.filter((e) => e.name !== "chromium");
+      executables = executables.filter((e) => e.name !== "chromium" && e.name !== "chromium-tip-of-tree");
     return executables;
   }
   suggestedBrowsersToInstall() {
-    return this.executables().filter((e) => e.installType !== "none" && e.type !== "tool").map((e) => e.name).join(", ");
+    const names = this.executables().filter((e) => e.installType !== "none").map((e) => e.name);
+    names.push(...chromiumAliases);
+    return names.sort().join(", ");
+  }
+  isChromiumAlias(name) {
+    return chromiumAliases.includes(name);
   }
   resolveBrowsers(aliases, options) {
     if (aliases.length === 0)
@@ -1298,15 +1316,20 @@ ${e.stack}`);
         faultyArguments.push(arg);
       else
         executables.push(executable);
-      if (executable?.browserName === "chromium")
+      if (executable?.browserName)
         executables.push(this.findExecutable("ffmpeg"));
     };
     for (const alias of aliases) {
-      if (alias === "chromium") {
+      if (alias === "chromium" || chromiumAliases.includes(alias)) {
         if (options.shell !== "only")
           handleArgument("chromium");
         if (options.shell !== "no")
           handleArgument("chromium-headless-shell");
+      } else if (alias === "chromium-tip-of-tree") {
+        if (options.shell !== "only")
+          handleArgument("chromium-tip-of-tree");
+        if (options.shell !== "no")
+          handleArgument("chromium-tip-of-tree-headless-shell");
       } else {
         handleArgument(alias);
       }
